package cadenasSeparadas;
import contenedores.*;
/**
 * Implementacin de un contenedor usando una tabla de dispersin
 * con prueba lineal.
 */
public class TablaDispersin implements Contenedor {
	private class Nodo {
		Object elemento;
		Nodo siguiente;

		public Nodo(Object e, Nodo sig) {
			elemento = e;
			siguiente = sig;
		}
	}

	Nodo elementos[];
	int numElementos = 0;

	/**
	 * Se inicializa la tabla con capacidad por defecto para
	 * 51 elementos
	 */
	public TablaDispersin() {
		this(51);
	}

	/**
	 * Se inicializa la tabla con la capacidad indicada
	 */
	public TablaDispersin(int capacidad) {
		elementos = new Nodo[capacidad];
	}

	/**
	 * Hace corresponder la pseudoclave de hash del (hashCode) objeto
	 * con una posicin del espacio de almacenamiento (elementos)
	 */
	private int disp(Object e) {
		return e.hashCode() % elementos.length;
	}

	// Clase auxiliar para contener informacin sobre la localizacin
	// de un objeto en la tabla
	private class ResultadoBsqueda {
		int pos;
		Nodo actual, anterior;
	}

	/**
	 * Devuelve la posicin de e en la tabla de dispersin, el nodo en
	 * que se encuentra y el anterior a ste
	 */
	private ResultadoBsqueda buscar(Object e) {
		ResultadoBsqueda resultado = new ResultadoBsqueda();
		// La clave debe estar en la lista encadenada indicado por su
		// direccin de dispersin
		resultado.pos = disp(e);
		resultado.actual = elementos[resultado.pos];
		resultado.anterior = null;
		while ((resultado.actual != null)
			&& (!(resultado.actual.elemento.hashCode() == e.hashCode()))) {
			resultado.anterior = resultado.actual;
			resultado.actual = resultado.actual.siguiente;
		}

		return resultado;
	}

	/* (non-Javadoc)
	 * @see contenedores.Contenedor#insertar(java.lang.Object)
	 */
	public void insertar(Object e) {
		// Se busca la clave y si no est se inserta en la lista indicada
		ResultadoBsqueda resultado = buscar(e);
		if (resultado.actual == null) {
			resultado.actual = new Nodo(e, elementos[resultado.pos]);
			elementos[resultado.pos] = resultado.actual;
			numElementos++;
		}
	}

	/* (non-Javadoc)
	 * @see contenedores.Contenedor#extraer(java.lang.Object)
	 */
	public void extraer(Object e) {
		// Se busca la clave y si est se quita de la lista indicada
		ResultadoBsqueda resultado = buscar(e);
		if (resultado.actual != null) {
			if (resultado.anterior == null)
				elementos[resultado.pos] = resultado.actual.siguiente;
			else
				resultado.anterior.siguiente = resultado.actual.siguiente;
			numElementos--;
		}
	}

	/* (non-Javadoc)
	 * @see contenedores.Contenedor#vaciar()
	 */
	public void vaciar() {
		for (int i = 0; i < elementos.length; i++)
			elementos[i] = null;
		numElementos = 0;
	}

	/* (non-Javadoc)
	 * @see contenedores.Contenedor#est(java.lang.Object)
	 */
	public boolean est(Object e) {
		ResultadoBsqueda resultado = buscar(e);
		return (resultado.actual != null);
	}

	/* (non-Javadoc)
	 * @see contenedores.Contenedor#esVaco()
	 */
	public boolean esVaco() {
		return (numElementos == 0);
	}

	/* (non-Javadoc)
	 * @see contenedores.Contenedor#tamao()
	 */
	public int tamao() {
		return numElementos;
	}
}
